@@ -7,7 +7,8 @@ from shutil import rmtree, copyfile
 class TreesappTester(unittest.TestCase):
     def setUp(self) -> None:
         from .testing_utils import get_test_data
-        self.num_procs = 4
+        from treesapp import utilities
+        self.num_procs = max(int(utilities.available_cpu_count()/2), 2)
         # FASTA files
         self.aa_test_fa = get_test_data("marker_test_suite.faa")
         self.nt_test_fa = get_test_data("marker_test_suite.fna")
@@ -93,11 +94,13 @@ class TreesappTester(unittest.TestCase):
                                 "--hmm_coverage", str(20),
                                 "--output", "./TreeSAPP_assign/",
                                 "--stringency", "relaxed",
-                                "--placement_summary", "max_lwr",
+                                "--hmm_coverage", str(80),
+                                "--query_coverage", str(80),
+                                "--placement_summary", "aelw",
                                 "--trim_align", "--overwrite", "--delete", "--svm"]
         assign.assign(assign_commands_list)
         lines = read_classification_table("./TreeSAPP_assign/final_outputs/classifications.tsv")
-        self.assertEqual(15, len(lines))
+        self.assertEqual(13, len(lines))
         self.assertTrue(os.path.isfile("./TreeSAPP_assign/final_outputs/marker_test_suite_classified.faa"))
 
         # Test nucleotide sequence input
@@ -107,7 +110,7 @@ class TreesappTester(unittest.TestCase):
                                 "--num_procs", str(self.num_procs),
                                 "--output", "./TreeSAPP_assign/",
                                 "--stringency", "strict",
-                                "--trim_align", "--overwrite", "--delete"]
+                                "--trim_align", "--overwrite", "--delete", "--silent"]
         assign.assign(assign_commands_list)
 
         assign_commands_list += ["--rel_abund",
@@ -127,6 +130,7 @@ class TreesappTester(unittest.TestCase):
 
     def test_colour(self):
         from treesapp.commands import colour
+        from treesapp.refpkg import ReferencePackage
         # Test creating iTOL files for a taxonomic rank
         colour_commands = ["-r", self.mcra_pkl, self.mcrb_pkl,
                            "-l", "family",
@@ -148,12 +152,21 @@ class TreesappTester(unittest.TestCase):
         colour_commands = ["-r", self.xmoa_pkl,
                            "-o", "./TreeSAPP_colour",
                            "--palette", "viridis",
-                           "--attribute", "Paralog"]
+                           "--attribute", "Paralog",
+                           "--unknown_colour", "gray"]
         colour(colour_commands)
         itol_strip_file = os.path.join("TreeSAPP_colour", "XmoA_Paralog_colour_strip.txt")
+        itol_colour_file = os.path.join("TreeSAPP_colour", "XmoA_Paralog_colours_style.txt")
+        # Check number of lines in the strip file
         self.assertTrue(os.path.isfile(itol_strip_file))
         with open(itol_strip_file) as strip_dat:
-            self.assertEqual(25, len(strip_dat.readlines()))
+            self.assertEqual(45, len(strip_dat.readlines()))
+        # Check number of lines in the styles file: one line for each reference sequence and the three-line header
+        xmoa_rp = ReferencePackage()
+        xmoa_rp.f__pkl = self.xmoa_pkl
+        xmoa_rp.slurp()
+        with open(itol_colour_file) as style_dat:
+            self.assertEqual(xmoa_rp.num_seqs + 3, len(style_dat.readlines()))
         return
 
     def test_create(self):
@@ -225,7 +238,7 @@ class TreesappTester(unittest.TestCase):
         return
 
     def test_evaluate(self):
-        from treesapp.commands import evaluate
+        from treesapp import commands
         from .testing_utils import get_test_data
         evaluate_command_list = ["--fastx_input", get_test_data("McrA_eval.faa"),
                                  "--refpkg_path", self.mcra_pkl,
@@ -235,10 +248,10 @@ class TreesappTester(unittest.TestCase):
                                  "--taxonomic_ranks", "class", "order",
                                  "-n", str(self.num_procs),
                                  "--trim_align"]
-        evaluate(evaluate_command_list)
+        commands.evaluate(evaluate_command_list)
 
         # Test rerunning of already processed data
-        evaluate(evaluate_command_list)
+        commands.evaluate(evaluate_command_list)
         self.assertTrue(True)
         return
 
@@ -327,11 +340,11 @@ class TreesappTester(unittest.TestCase):
         test_refpkg.slurp()
         self.assertIsInstance(test_refpkg.feature_annotations, dict)
         self.assertTrue('Function' in test_refpkg.feature_annotations)
-        self.assertEqual(5, len(test_refpkg.feature_annotations["Function"]))
+        self.assertEqual(6, len(test_refpkg.feature_annotations["Function"]))
         self.assertIsInstance(test_refpkg.feature_annotations["Function"][0], CladeAnnotation)
         for ca in test_refpkg.feature_annotations["Function"]:
             if ca.name == "Short-chain alkane oxidation":
-                self.assertEqual(2, len(ca.members))
+                self.assertEqual(5, len(ca.members))
 
         # Test view with a complex attribute (feature_annotations)
         view_command_list = ["view",
@@ -411,10 +424,13 @@ class TreesappTester(unittest.TestCase):
         return
 
     # def test_tmp(self):
-    #     from treesapp.commands import create
-    #     base_dir = "/home/connor/Bioinformatics/Hallam_projects/RefPkgs/"
-    #     cmd = "".format(base_dir)
-    #     create(cmd.split())
+    #     from treesapp import phylo_cluster
+    #     base_dir = "/media/connor/Rufus/Gene_Centric_Guide/clustering_experiments"
+    #     cmd = "--refpkg_path {0}/clustering_refpkgs/RecA_build.pkl" \
+    #           " --assign_out {0}/length_100/proteome_classified/" \
+    #           " -o {0}/length_100/phylotu_outputs/RecA_phylotus_dn_psc_species/" \
+    #           " --mode de_novo --pre_cluster psc".format(base_dir)
+    #     phylo_cluster.cluster_phylogeny(cmd.split())
     #     return
 
 
