@@ -7,7 +7,7 @@ import shutil
 from copy import deepcopy
 from ete3 import Tree, TreeNode
 
-from .testing_utils import get_test_data
+from .testing_utils import get_test_data, random_ete_tree
 
 
 class PhyloClusterTester(unittest.TestCase):
@@ -53,19 +53,24 @@ class PhyloClusterTester(unittest.TestCase):
                          p_clust.stage_output_dir)
         self.assertTrue(os.path.isdir(p_clust.stage_output_dir))
         self.assertTrue("mmseqs" in p_clust.executables)
+        return
 
+    def test_load_sample_placement_files(self):
+        from treesapp import phylo_cluster
+        p_clust = phylo_cluster.PhyloClust()
         # Simulate failure by de novo clustering with JPlace file
         with pytest.raises(SystemExit):
             p_clust.load_args(p_clust.arg_parser.parse_args(["--refpkg_path", self.refpkg.f__pkl,
                                                              "--mode", "de_novo",
                                                              "--jplace", get_test_data("epa_result.jplace")]))
+            p_clust.load_sample_placement_files()
         return
 
     def test_partition_nodes(self):
         from treesapp.phylo_cluster import PhyloClust
         p_clust = PhyloClust()
         p_clust.alpha = 0.1
-        node_paritions = p_clust.partition_nodes(tree=deepcopy(self.mock_tree))
+        node_paritions = p_clust.partition_nodes(tree=deepcopy(self.mock_tree), alpha=p_clust.alpha)
         # Ensure all the tree's nodes are present in the partitions
         count = 0
         for st in node_paritions.values():
@@ -76,7 +81,7 @@ class PhyloClusterTester(unittest.TestCase):
 
         # Test each leaf in a separate cluster
         p_clust.alpha = 0.2
-        self.assertEqual(3, len(p_clust.partition_nodes(tree=deepcopy(self.mock_tree))))
+        self.assertEqual(3, len(p_clust.partition_nodes(tree=deepcopy(self.mock_tree), alpha=p_clust.alpha)))
         return
 
     def test_build_edge_node_index(self):
@@ -93,7 +98,7 @@ class PhyloClusterTester(unittest.TestCase):
         p_clust = PhyloClust()
         p_clust.alpha = 0.1
         # Test empty
-        edge_clusters = p_clust.split_node_partition_edges({})
+        edge_clusters = p_clust.split_node_partition_edges(p_clust.alpha, {})
         self.assertEqual(0, len(edge_clusters))
 
         a, b, c = TreeNode(name='A', dist=0.2), TreeNode(name='B', dist=0.4), TreeNode(name='0', dist=0.1)
@@ -101,7 +106,7 @@ class PhyloClusterTester(unittest.TestCase):
         c.children = [a]
         a.up = c
         self.assertEqual(2, len(mock_partitions))
-        edge_clusters = p_clust.split_node_partition_edges(mock_partitions)
+        edge_clusters = p_clust.split_node_partition_edges(p_clust.alpha, mock_partitions)
         self.assertEqual(3, len(edge_clusters))
         return
 
@@ -116,8 +121,35 @@ class PhyloClusterTester(unittest.TestCase):
             self.assertTrue(1 > max(dists))
 
         # Test grouping by a deeper taxonomic relationship, family
+        p_clust.normalize = True
         p_clust.tax_rank = "family"
-        p_clust.group_rel_dists(self.taxa_tree, self.refpkg.taxa_trie)
+        rel_dists = p_clust.group_rel_dists(self.taxa_tree, self.refpkg.taxa_trie)
+        self.assertEqual(5, len(rel_dists))
+        self.assertEqual(403, len(sum(rel_dists.values(), [])))
+        return
+
+    def test_calculate_distance_threshold(self):
+        from treesapp import phylo_cluster
+        p_clust = phylo_cluster.PhyloClust()
+        alpha = p_clust.calculate_distance_threshold(taxa_tree=self.taxa_tree,
+                                                     taxonomy=self.refpkg.taxa_trie)
+        self.assertTrue(0.04 < round(alpha, 3) <= 0.08)
+
+        # Define a subtree for a group of related taxa
+        def clade_member(node: TreeNode) -> bool:
+            t = getattr(node, "taxon")
+            if "g__Methanosphaera" in [r.prefix_taxon() for r in t.lineage()]:
+                return True
+            return False
+
+        subtree_leaves = self.taxa_tree.get_leaves(is_leaf_fn=clade_member)
+        small_tree = subtree_leaves[0].get_common_ancestor(subtree_leaves)
+        self.assertEqual(4, len(small_tree))
+
+        self.assertEqual(-1, p_clust.calculate_distance_threshold(small_tree,
+                                                                  self.refpkg.taxa_trie,
+                                                                  override_rank="class"))
+        p_clust.calculate_distance_threshold(small_tree, self.refpkg.taxa_trie, override_rank="species")
         return
 
     def test_match_edges_to_clusters(self):
@@ -147,6 +179,148 @@ class PhyloClusterTester(unittest.TestCase):
         p_clust.alpha = 0.05
         p_clust.define_tree_clusters(tree=deepcopy(self.taxa_tree))
         self.assertTrue(len(self.taxa_tree) <= len(p_clust.cluster_index) <= sum([1 for _ in self.taxa_tree.traverse()]))
+
+        p_clust.define_tree_clusters(tree=deepcopy(self.taxa_tree), override_alpha=0.1)
+        self.assertEqual(0.05, p_clust.alpha)
+        return
+
+    def test_infer_cluster_phylogeny(self):
+        from treesapp import phylo_cluster
+        from treesapp import fasta
+        p_clust = phylo_cluster.PhyloClust()
+        p_clust.load_args(p_clust.get_arguments(["--refpkg_path", self.refpkg.f__pkl,
+                                                 "--assign_output", get_test_data("test_output_TarA"),
+                                                 "--output", self.tmp_dir,
+                                                 "--alpha", str(0.4)]))
+        q_seqs = fasta.FASTA(get_test_data("McrA_eval.faa"))
+        q_seqs.load_fasta()
+        _fa_input, ete_tree = phylo_cluster.infer_cluster_phylogeny(fa_input=q_seqs.file,
+                                                                    executables=p_clust.executables,
+                                                                    output_dir=p_clust.stage_output_dir)
+        # Test for node names
+        ex_name = "KKH90701"
+        self.assertTrue(ex_name in ete_tree.get_leaf_names())
+        self.assertEqual(os.path.join(self.tmp_dir, 'intermediates', 'load_pqueries/'), p_clust.stage_output_dir)
+
+        # Test for number of nodes
+        self.assertEqual(q_seqs.n_seqs(), len(ete_tree))
+        return
+
+    def test_assign_pqueries_to_leaf_clusters(self):
+        from treesapp import phylo_cluster
+        from treesapp import file_parsers
+        from treesapp import assign
+        from treesapp import seq_clustering
+        p_clust = phylo_cluster.PhyloClust()
+        pqueries = file_parsers.load_classified_sequences_from_assign_output(assign_output_dir=get_test_data("test_output_TarA"),
+                                                                             assigner_cls=assign.Assigner(),
+                                                                             refpkg_name="McrA")["McrA"]
+        pquery_names = []
+        for pq in pqueries:
+            pquery_names.append(pq.place_name)
+            setattr(pq, "sample_name", "test")
+
+        i = 0
+        spread = 3
+        precluster_map = {}
+        while pquery_names:
+            p_otu = phylo_cluster.PhylOTU(name=str(i))
+            p_otu.tree_node = random_ete_tree(pquery_names[0:spread])
+            p_clust.cluster_index[str(i)] = p_otu
+            if i % 2 == 0:
+                mock_cluster = seq_clustering.Cluster(pquery_names[0])
+                mock_cluster.members = pqueries[i*spread:(i*spread)+spread]
+                precluster_map[i] = {str(i): mock_cluster}
+            i += 1
+            pquery_names = pquery_names[spread:]
+        self.assertEqual([0, 2], list(precluster_map.keys()))
+        self.assertEqual(4, len(p_clust.cluster_index))
+
+        p_clust.assign_pqueries_to_leaf_clusters(pqueries=pqueries,
+                                                 cluster_map=precluster_map)
+        self.assertEqual({'0': 3, '1': 3, '2': 3, '3': 2},
+                         p_clust.sample_mat["test"])
+
+        return
+
+    def test_format_precluster_map(self):
+        from treesapp import phylo_cluster
+        precluster_map = {0: "Cluster", 1: "Cluster", 2: "Cluster"}
+        phylo_groups = phylo_cluster.format_precluster_map(cluster_method="align", precluster_map=precluster_map)
+        self.assertEqual(1, len(phylo_groups))
+        self.assertEqual(3, len(phylo_groups[0]))
+        return
+
+    def test_de_novo_phylo_clusters(self):
+        from treesapp import phylo_cluster
+        from treesapp import rel_evo_dist
+        p_clust = phylo_cluster.PhyloClust()
+        p_clust.load_args(p_clust.get_arguments(["--refpkg_path", self.refpkg.f__pkl,
+                                                 "--assign_output", get_test_data("test_output_TarA"),
+                                                 "--output", self.tmp_dir,
+                                                 "--alpha", str(0.4)]))
+        p_clust.load_sample_placement_files()
+        # Test with placement space clustering
+        taxa_tree = p_clust.ref_pkg.taxonomically_label_tree()
+        red_tree = rel_evo_dist.RedTree()
+        red_tree.decorate_rel_dist(taxa_tree)
+
+        phylo_cluster.de_novo_phylo_clusters(p_clust, taxa_tree, cluster_method="psc")
+        self.assertEqual(11, len(p_clust.clustered_pqueries))
+        self.assertEqual(len(p_clust.clustered_pqueries),
+                         len(set([pq.p_otu for pq in p_clust.clustered_pqueries])))
+        # Test with pairwise sequence clustering
+        p_clust.cluster_index.clear()
+        p_clust.clustered_pqueries.clear()
+        p_clust._edges_to_cluster_index.clear()
+        phylo_cluster.de_novo_phylo_clusters(p_clust, taxa_tree, cluster_method="align", drep_id=0.8)
+        self.assertEqual(10, len(set([pq.p_otu for pq in p_clust.clustered_pqueries])))
+
+        phylo_cluster.de_novo_phylo_clusters(p_clust, taxa_tree, cluster_method="psc", phylo_group=None)
+
+        return
+
+    def test_subtree_finder(self):
+        from treesapp import phylo_cluster
+        target_leaves = {'9_McrA', '6_McrA', '7_McrA',
+                         '37_McrA', '49_McrA', '27_McrA',
+                         '191_McrA', '192_McrA', '194_McrA'}
+        subtree_size = 3
+        sub_root = phylo_cluster.subtree_finder(ref_tree=self.refpkg.get_ete_tree(),
+                                                leaf_nodes=target_leaves,
+                                                tree_size=subtree_size)
+        self.assertTrue(set(sub_root.get_leaf_names()).issubset(target_leaves))
+        self.assertEqual(subtree_size, len(sub_root.get_leaf_names()))
+        return
+
+    def test_select_subtree_sequences(self):
+        from treesapp import phylo_cluster
+        from treesapp.seq_clustering import Cluster
+        from treesapp.phylo_seq import PQuery, PhyloPlace
+        mock_cluster = Cluster("TarA")
+        test_placement_edges = [62, 60, 65, 63]
+        mock_placements = []
+        while test_placement_edges:
+            pplace = PhyloPlace()
+            pplace.edge_num = test_placement_edges.pop()
+            mock_placements.append(pplace)
+        while mock_placements:
+            pq = PQuery()
+            pq.consensus_placement = mock_placements.pop()
+            mock_cluster.members.append(pq)
+
+        n = 8
+        ref_fasta = phylo_cluster.select_subtree_sequences(ref_pkg=self.refpkg,
+                                                           clusters=[mock_cluster],
+                                                           subtree_size=n)
+        self.assertEqual(n+1, ref_fasta.n_seqs())
+        return
+
+    def test_get_outgroup(self):
+        from treesapp import phylo_cluster
+        outgroup = phylo_cluster.get_outgroup(tree=self.mock_tree, target="D")
+        self.assertIsInstance(outgroup, TreeNode)
+        self.assertEqual('A', outgroup.name)
         return
 
     def test_cluster_phylogeny(self):
@@ -157,66 +331,26 @@ class PhyloClusterTester(unittest.TestCase):
                                self.refpkg.f__pkl, get_test_data(os.path.join("refpkgs", "McrB_build.pkl")),
                                "--jplace", get_test_data("epa_result.jplace")])
 
-        # Test input is a single JPlace file
-        cluster_phylogeny(["--refpkg_path", self.refpkg.f__pkl,
-                           "--jplace", get_test_data("epa_result.jplace"),
-                           "--output", self.tmp_dir])
-
         # Test input is a treesapp assign output directory
         cluster_phylogeny(["--refpkg_path", self.refpkg.f__pkl,
                            "--assign_output", get_test_data("test_output_TarA"),
                            "--output", self.tmp_dir,
                            "--alpha", str(0.4),
-                           "--mode", "de_novo"])
+                           "--mode", "de_novo",
+                           "--delete"])
+        self.assertFalse(os.path.isdir(os.path.join(self.tmp_dir, "intermediates")))
+
+        # Test input is a single JPlace file
+        cluster_phylogeny(["--refpkg_path", self.refpkg.f__pkl,
+                           "--jplace", get_test_data("epa_result.jplace"),
+                           "--output", self.tmp_dir])
+
         self.assertTrue(os.path.isfile(os.path.join(self.tmp_dir, "final_outputs", "phylotu_taxa.tsv")))
         self.assertTrue(os.path.isfile(os.path.join(self.tmp_dir, "final_outputs", "phylotu_matrix.tsv")))
         self.assertTrue(os.path.isfile(os.path.join(self.tmp_dir, "final_outputs", "phylotu_pquery_assignments.tsv")))
-        return
-
-    def test_dereplicate_by_clustering(self):
-        from treesapp.phylo_cluster import dereplicate_by_clustering
-        from treesapp import fasta
-        from treesapp.utilities import fetch_executable_path
-        test_fa = fasta.FASTA(get_test_data("PuhA.fa"))
-        test_fa.load_fasta()
-        cluster_map = dereplicate_by_clustering(fasta_inst=test_fa,
-                                                prop_similarity=0.799,
-                                                mmseqs_exe=fetch_executable_path("mmseqs", treesapp_dir="./"),
-                                                tmp_dir=self.tmp_dir)
-        self.assertEqual(len(cluster_map), test_fa.n_seqs())
-        return
-
-    def test_de_novo_phylogeny_from_queries(self):
-        from treesapp.phylo_cluster import de_novo_phylogeny_from_queries
-        from treesapp.fasta import FASTA
-        from treesapp import phylo_cluster
-        p_clust = phylo_cluster.PhyloClust()
-        p_clust.load_args(p_clust.get_arguments(["--refpkg_path", self.refpkg.f__pkl,
-                                                 "--assign_output", get_test_data("test_output_TarA"),
-                                                 "--output", self.tmp_dir,
-                                                 "--alpha", str(0.4)]))
-        q_seqs = FASTA(get_test_data("McrA_eval.faa"))
-        q_seqs.load_fasta()
-        ete_tree = de_novo_phylogeny_from_queries(phylo_clust=p_clust, fasta_inst=q_seqs)
-        # Test for node names
-        ex_name = "KKH90701  coded_by=13826..15538,organism=Methanosarcina mazei,definition=methyl-coenzyme M reductase"
-        self.assertTrue(ex_name in ete_tree.get_leaf_names())
-        self.assertEqual(0, len(set(ete_tree.get_leaf_names()).difference(q_seqs.get_seq_names())))
-        self.assertEqual(os.path.join(self.tmp_dir, 'intermediates', 'load_pqueries/'), p_clust.stage_output_dir)
-
-        # Test for number of nodes
-        self.assertEqual(q_seqs.n_seqs(), len(ete_tree))
-        return
-
-    def test_de_novo_phylo_clusters(self):
-        from treesapp import phylo_cluster
-        p_clust = phylo_cluster.PhyloClust()
-        p_clust.load_args(p_clust.get_arguments(["--refpkg_path", self.refpkg.f__pkl,
-                                                 "--assign_output", get_test_data("test_output_TarA"),
-                                                 "--output", self.tmp_dir,
-                                                 "--alpha", str(0.4)]))
-        phylo_cluster.de_novo_phylo_clusters(p_clust, 0.9)
-
+        with open(os.path.join(self.tmp_dir, "final_outputs", "phylotu_pquery_assignments.tsv")) as asns:
+            lines = asns.readlines()
+        self.assertEqual(13, len(lines))
         return
 
 
