@@ -1,21 +1,20 @@
-#!/usr/bin/env python3
-
-__author__ = 'Connor Morgan-Lang'
-
 import os
 import logging
 import sys
 import re
 import shutil
 
-from treesapp.external_command_interface import launch_write_command
 from treesapp import refpkg
-from treesapp.entrez_utils import EntrezRecord
+from treesapp import entrez_utils as ez_utils
 from treesapp import fasta
 from treesapp import classy
 from treesapp import taxonomic_hierarchy
 from treesapp import assign
 from treesapp import file_parsers
+from treesapp import logger
+from treesapp import graftm_utils
+
+LOGGER = logging.getLogger(logger.logger_name())
 
 
 def parse_distances(classification_lines):
@@ -69,7 +68,7 @@ def determine_containment(marker_eval_inst: classy.Evaluator):
         correct = 0
         incorrect = 0
         too_shallow = 0
-        parse_depth = depth-1
+        parse_depth = depth - 1
         for assignments in rank_assigned_dict[lowest_rank]:
             for classified in assignments:
                 status = 0  # 0 == incorrect, 1 == correct
@@ -131,39 +130,24 @@ def map_headers_to_lineage(assignments: dict, ref_sequences: dict) -> dict:
             lineage_assignments[refpkg_name][assigned_lineage] = list()
             for query in classified_headers:
                 mapped = False
-                for _, ref_seq in ref_sequences.items():  # type: (int, EntrezRecord)
+                for _, ref_seq in ref_sequences.items():  # type: (int, ez_utils.EntrezRecord)
                     if ref_seq.accession == query:
                         lineage_assignments[refpkg_name][assigned_lineage].append(ref_seq.lineage)
                         mapped = True
                         break
                 if not mapped:
-                    logging.error("Unable to map classified sequence '" + query + "' to a lineage.\n")
+                    LOGGER.error("Unable to map classified sequence '" + query + "' to a lineage.\n")
                     sys.exit(3)
             if len(lineage_assignments[refpkg_name][assigned_lineage]) > len(classified_headers):
-                logging.error(str(len(classified_headers)) + " accessions mapped to " +
-                              str(len(lineage_assignments[refpkg_name][assigned_lineage])) + " lineages.\n")
+                LOGGER.error(str(len(classified_headers)) + " accessions mapped to " +
+                             str(len(lineage_assignments[refpkg_name][assigned_lineage])) + " lineages.\n")
                 sys.exit(21)
             elif len(lineage_assignments[refpkg_name][assigned_lineage]) < len(classified_headers):
-                logging.debug(str(len(classified_headers)) + " accessions mapped to " +
-                              str(len(lineage_assignments[refpkg_name][assigned_lineage])) + " lineages.\n")
+                LOGGER.debug(str(len(classified_headers)) + " accessions mapped to " +
+                             str(len(lineage_assignments[refpkg_name][assigned_lineage])) + " lineages.\n")
     return lineage_assignments
 
 
-def get_unclassified_rank(pos, split_lineage):
-    """
-    Recursive function to retrieve the first rank at which the
-
-    :param pos:
-    :param split_lineage:
-    :return:
-    """
-    if re.search("unclassified", split_lineage[pos], re.IGNORECASE):
-        return pos
-    else:
-        pos = get_unclassified_rank(pos+1, split_lineage)
-    return pos
-
-
 def check_lineage_compatibility(lineage_map: dict, taxonomy: taxonomic_hierarchy.TaxonomicHierarchy) -> None:
     """Ensures that all the lineages are rooted."""
     unrooted = False
@@ -174,7 +158,7 @@ def check_lineage_compatibility(lineage_map: dict, taxonomy: taxonomic_hierarchy
             unrooted = True
             lineage_map[num_id] = taxonomy.root_taxon + taxonomy.lin_sep + lineage
     if unrooted:
-        logging.warning("Unrooted lineages have been rooted.\n")
+        LOGGER.warning("Unrooted lineages have been rooted.\n")
     return
 
 
@@ -201,7 +185,7 @@ def get_testable_lineages_for_rank(ref_lineage_map: dict, query_lineage_map: dic
         optimal_lca_taxonomy = "; ".join(lineage.split("; ")[:-1])
         if optimal_lca_taxonomy not in ["; ".join(tl.split("; ")[:-1]) for tl in unique_ref_lineages
                                         if tl != lineage]:
-            logging.debug("Optimal placement target '" + optimal_lca_taxonomy + "' not in pruned tree.\n")
+            LOGGER.debug("Optimal placement target '" + optimal_lca_taxonomy + "' not in pruned tree.\n")
         else:
             lineages.append(lineage)
     return lineages
@@ -238,7 +222,7 @@ def pick_taxonomic_representatives(ref_seqs: dict, taxonomic_filter_stats: dict,
             good_classified_lineages[ref_seq.lineage].append(ref_seq.accession)
 
     if taxonomic_filter_stats["Unclassified"] == len(ref_seqs):
-        logging.error("All sequences provided are derived from uncultured, unclassified organisms.\n")
+        LOGGER.error("All sequences provided are derived from uncultured, unclassified organisms.\n")
         sys.exit(21)
 
     # In order to maintain consistency among multiple runs with the same input
@@ -268,10 +252,10 @@ def pick_taxonomic_representatives(ref_seqs: dict, taxonomic_filter_stats: dict,
 
     taxonomic_filter_stats["Unique_taxa"] += len(dereplicated_lineages)
 
-    logging.debug("Representative sequence stats:\n\t" +
-                  "Maximum representative sequences for a taxon " + str(taxonomic_filter_stats["Max"]) + "\n\t" +
-                  "Minimum representative sequences for a taxon " + str(taxonomic_filter_stats["Min"]) + "\n\t" +
-                  "Mean representative sequences for a taxon " + str(taxonomic_filter_stats["Mean"]) + "\n")
+    LOGGER.debug("Representative sequence stats:\n\t" +
+                 "Maximum representative sequences for a taxon " + str(taxonomic_filter_stats["Max"]) + "\n\t" +
+                 "Minimum representative sequences for a taxon " + str(taxonomic_filter_stats["Min"]) + "\n\t" +
+                 "Mean representative sequences for a taxon " + str(taxonomic_filter_stats["Mean"]) + "\n")
 
     return dereplicated_lineages, taxonomic_filter_stats
 
@@ -309,7 +293,7 @@ def select_rep_seqs(deduplicated_assignments: dict, test_sequences: dict,
             try:
                 deduplicated_fasta_dict[accession] = test_seq_lookup[accession]
             except KeyError:
-                logging.error("Unable to find accession (" + accession + ") in accession-lineage map\n")
+                LOGGER.error("Unable to find accession (" + accession + ") in accession-lineage map\n")
                 sys.exit(21)
     return deduplicated_fasta_dict
 
@@ -318,49 +302,13 @@ def map_seqs_to_lineages(accession_lineage_map, deduplicated_fasta_dict):
     seq_taxa_map = dict()
     for seq_name in deduplicated_fasta_dict:
         if seq_name not in accession_lineage_map:
-            logging.error("Unable to find matching key for '" + seq_name + "' in accession_lineage_map.\n")
+            LOGGER.error("Unable to find matching key for '" + seq_name + "' in accession_lineage_map.\n")
             sys.exit(21)
         else:
             seq_taxa_map[seq_name] = accession_lineage_map[seq_name]
     return seq_taxa_map
 
 
-# def filter_queries_by_taxonomy(taxonomic_lineages):
-#     """
-#     Removes queries with duplicate taxa - to prevent the taxonomic composition of the input
-#     from biasing the accuracy to over- or under-perform by classifying many sequences from very few groups.
-#     Also removes taxonomies with "*nclassified" in their lineage
-#
-#     :param taxonomic_lineages: A list of lineages that need to be filtered
-#     :return: A list that contains no more than 3 of each query taxonomy (arbitrarily normalized) and counts
-#     """
-#     unclassifieds = 0
-#     classified = 0
-#     unique_query_taxonomies = 0
-#     lineage_enumerator = dict()
-#     normalized_lineages = list()
-#     for query_taxonomy in sorted(taxonomic_lineages):
-#         can_classify = True
-#         if re.search("unclassified", query_taxonomy, re.IGNORECASE):
-#             # Remove taxonomic lineages that are unclassified at the Phylum level or higher
-#             unclassified_depth = get_unclassified_rank(0, query_taxonomy.split("; "))
-#             if unclassified_depth <= 3:
-#                 can_classify = False
-#
-#         if can_classify:
-#             if query_taxonomy not in lineage_enumerator:
-#                 lineage_enumerator[query_taxonomy] = 0
-#                 classified += 1
-#             if lineage_enumerator[query_taxonomy] < 3:
-#                 normalized_lineages.append(query_taxonomy)
-#             lineage_enumerator[query_taxonomy] += 1
-#         else:
-#             unclassifieds += 1
-#     unique_query_taxonomies += len(lineage_enumerator)
-#
-#     return normalized_lineages, unclassifieds, classified, unique_query_taxonomies
-
-
 def run_clade_exclusion_treesapp(tt_obj: classy.TaxonTest, taxon_rep_seqs, ref_pkg, num_threads, executables,
                                  trim_align=False, fresh=False, molecule_type="prot", min_seq_length=30) -> None:
     if not os.path.isfile(tt_obj.classification_table):
@@ -374,22 +322,17 @@ def run_clade_exclusion_treesapp(tt_obj: classy.TaxonTest, taxon_rep_seqs, ref_p
                        "--refpkg_dir", os.path.dirname(ce_refpkg.f__pkl),
                        "-m", molecule_type, "-n", str(num_threads),
                        "--min_seq_length", str(min_seq_length),
-                       "--overwrite", "--delete"]
+                       "--overwrite", "--delete", "--silent"]
         if trim_align:
             assign_args.append("--trim_align")
 
-        # TODO: Pause logging just to console and continue writing to log file
-        # Option 1. No logging to console or file
-        cl_log = logging.getLogger()
-        cl_log.disabled = True
         try:
             assign.assign(assign_args)
         except:  # Just in case treesapp assign fails, just continue
             pass
-        cl_log.disabled = False
         if not os.path.isfile(tt_obj.classification_table):
             # The TaxonTest object is maintained for record-keeping (to track # queries & classifieds)
-            logging.warning("TreeSAPP did not generate output for '{}'. Skipping.\n".format(tt_obj.lineage))
+            LOGGER.warning("TreeSAPP did not generate output for '{}'. Skipping.\n".format(tt_obj.lineage))
             shutil.rmtree(tt_obj.classifications_root)
             return
     else:
@@ -407,10 +350,10 @@ def run_clade_exclusion_treesapp(tt_obj: classy.TaxonTest, taxon_rep_seqs, ref_p
         tt_obj.filter_assignments(ref_pkg.prefix)
         tt_obj.distances = parse_distances(assigned_lines)
     else:
-        logging.error("{} is missing from output directory '{}'\n"
-                      "Please remove this directory and re-run.\n"
-                      "".format(os.path.basename(tt_obj.classification_table),
-                                os.path.dirname(tt_obj.classification_table)))
+        LOGGER.error("{} is missing from output directory '{}'\n"
+                     "Please remove this directory and re-run.\n"
+                     "".format(os.path.basename(tt_obj.classification_table),
+                               os.path.dirname(tt_obj.classification_table)))
         sys.exit(21)
     return
 
@@ -424,103 +367,32 @@ def run_clade_exclusion_graftm(tt_obj: classy.TaxonTest, taxon_rep_seqs, ref_pkg
     gpkg_tax_ids_file = gpkg_refpkg_path + ref_pkg.prefix + "_taxonomy.csv"
     if not os.path.isfile(tt_obj.classification_table):
         # Copy reference files, then exclude all clades belonging to the taxon being tested
-        output_paths = prep_graftm_ref_files(tmp_dir=tt_obj.intermediates_dir,
-                                             target_clade=tt_obj.lineage,
-                                             ref_pkg=ref_pkg,
-                                             executables=executables)
+        output_paths = graftm_utils.prep_graftm_ref_files(tmp_dir=tt_obj.intermediates_dir,
+                                                          target_clade=tt_obj.lineage,
+                                                          ref_pkg=ref_pkg,
+                                                          executables=executables)
 
         if not os.path.isdir(tt_obj.refpkg_path):
-            build_graftm_package(gpkg_path=tt_obj.refpkg_path,
-                                 tax_file=output_paths["filtered_tax_ids"],
-                                 mfa_file=output_paths["filtered_mfa"],
-                                 fa_file=output_paths["filtered_fasta"],
-                                 threads=num_threads)
+            graftm_utils.build_graftm_package(gpkg_path=tt_obj.refpkg_path,
+                                              tax_file=output_paths["filtered_tax_ids"],
+                                              mfa_file=output_paths["filtered_mfa"],
+                                              fa_file=output_paths["filtered_fasta"],
+                                              threads=num_threads)
         # Write the query sequences
         fasta.write_new_fasta(taxon_rep_seqs, tt_obj.test_query_fasta)
 
-        graftm_classify(tt_obj.test_query_fasta, tt_obj.refpkg_path, tt_obj.classifications_root,
-                        num_threads, graftm_classifier)
+        graftm_utils.run_graftm_graft(input_path=tt_obj.test_query_fasta, gpkg_path=tt_obj.refpkg_path,
+                                      output_dir=tt_obj.classifications_root,
+                                      num_threads=num_threads, classifier=graftm_classifier)
 
         if not os.path.isfile(tt_obj.classification_table):
             # The TaxonTest object is maintained for record-keeping (to track # queries & classifieds)
-            logging.warning("GraftM did not generate output for " + tt_obj.lineage + ". Skipping.\n")
+            LOGGER.warning("GraftM did not generate output for " + tt_obj.lineage + ". Skipping.\n")
             shutil.rmtree(tt_obj.intermediates_dir)
 
-    tt_obj.taxonomic_tree = file_parsers.grab_graftm_taxa(gpkg_tax_ids_file)
-    graftm_assignments = file_parsers.read_graftm_classifications(tt_obj.classification_table)
+    tt_obj.taxonomic_tree = graftm_utils.grab_graftm_taxa(gpkg_tax_ids_file)
+    graftm_assignments = graftm_utils.read_graftm_classifications(tt_obj.classification_table)
     tt_obj.assignments = {ref_pkg.prefix: graftm_assignments}
     tt_obj.filter_assignments(ref_pkg.prefix)
 
     return
-
-
-def prep_graftm_ref_files(ref_pkg: refpkg.ReferencePackage, tmp_dir: str, target_clade: str, executables: dict) -> dict:
-    """
-    From the original TreeSAPP reference package files, the necessary GraftM create input files are generated
-    with all reference sequences related to the target_taxon removed from the multiple sequence alignment,
-    unaligned reference FASTA file and the tax_ids file.
-
-    :param tmp_dir:  Path to write the intermediate files with target references removed
-    :param target_clade: Taxonomic lineage of the clade that is being excluded from the reference package.
-    :param ref_pkg: A ReferencePackage instance for the reference package being tested
-    :param executables: Dictionary of paths to dependency executables indexed by their names. Must include:
-         'hmmbuild', 'FastTree' and 'raxml-ng'.
-    :return: A dictionary providing paths to output files
-    """
-    # GraftM refpkg input paths:
-    output_paths = {"filtered_tax_ids": os.path.join(tmp_dir, ref_pkg.prefix + "_lineage_ids.txt"),
-                    "filtered_mfa": os.path.join(tmp_dir, ref_pkg.prefix + ".mfa"),
-                    "filtered_fasta": os.path.join(tmp_dir, ref_pkg.prefix + ".fa")}
-
-    ce_refpkg = ref_pkg.clone(clone_path=tmp_dir + ref_pkg.prefix + ref_pkg.refpkg_suffix)
-
-    ce_refpkg.exclude_clade_from_ref_files(tmp_dir=tmp_dir, executables=executables, target_clade=target_clade)
-
-    # Write the lineage_ids file
-    lineage_info = []
-    for ref_leaf in ce_refpkg.generate_tree_leaf_references_from_refpkg():
-        lineage_info.append("{}_{}\t{}".format(ref_leaf.number, ce_refpkg.prefix, ref_leaf.lineage))
-
-    with open(output_paths["filtered_tax_ids"], 'w') as taxa_handler:
-        taxa_handler.write("\n".join(lineage_info) + "\n")
-
-    # Create and write the unaligned fasta file
-    ce_fasta = ce_refpkg.get_fasta()  # type: fasta.FASTA
-    fasta.write_new_fasta(fasta_dict=ce_fasta.fasta_dict, fasta_name=output_paths["filtered_mfa"])
-    ce_fasta.unalign()
-    fasta.write_new_fasta(fasta_dict=ce_fasta.fasta_dict, fasta_name=output_paths["filtered_fasta"])
-    return output_paths
-
-
-def build_graftm_package(gpkg_path: str, tax_file: str, mfa_file: str, fa_file: str, threads: int):
-    create_command = ["graftM", "create"]
-    create_command += ["--threads", str(threads)]
-    create_command += ["--alignment", mfa_file]
-    create_command += ["--sequences", fa_file]
-    create_command += ["--taxonomy", tax_file]
-    create_command += ["--output", gpkg_path]
-    create_command.append("--force")
-
-    logging.debug("Command used:\n" + ' '.join(create_command) + "\n")
-    launch_write_command(create_command, False)
-
-
-def graftm_classify(test_rep_taxa_fasta, gpkg_path, output_dir, threads, tool):
-    classify_command = ["graftM", "graft"]
-    classify_command += ["--forward", test_rep_taxa_fasta]
-    classify_command += ["--graftm_package", gpkg_path]
-    classify_command += ["--threads", str(threads)]
-    if tool == "graftm":
-        classify_command += ["--assignment_method", "pplacer"]
-        classify_command += ["--search_method", "hmmsearch"]
-    elif tool == "diamond":
-        classify_command += ["--assignment_method", "diamond"]
-        classify_command += ["--search_method", "diamond"]
-    classify_command += ["--output_directory", output_dir]
-    classify_command += ["--input_sequence_type", "aminoacid"]
-    classify_command.append("--force")
-
-    logging.debug("Command used:\n" + ' '.join(classify_command) + "\n")
-    launch_write_command(classify_command, False)
-
-    return
