@@ -1,7 +1,5 @@
 #!/usr/bin/env python3
 
-__author__ = 'Connor Morgan-Lang'
-
 import os
 import shutil
 import sys
@@ -12,25 +10,28 @@ from glob import glob
 from numpy import sqrt
 
 from treesapp import file_parsers
-from treesapp import wrapper
 from treesapp import utilities
 from treesapp import assign
+from treesapp import logger
+from treesapp import graftm_utils
 from treesapp.phylo_seq import assignments_to_pqueries
 from treesapp import refpkg as ts_ref_pkg
 from treesapp.fasta import get_headers, register_headers
-from treesapp.classy import prep_logging, get_header_info
+from treesapp.classy import get_header_info
 from treesapp.entrez_utils import EntrezRecord, get_multiple_lineages
 from treesapp import lca_calculations as ts_lca
 from treesapp.treesapp_args import TreeSAPPArgumentParser
 from treesapp.taxonomic_hierarchy import TaxonomicHierarchy
-from treesapp.training_utils import bin_headers, QuerySequence
+from treesapp import training_utils
+
+LOGGER = logging.getLogger(logger.logger_name())
 
 
 class ConfusionTest:
     def __init__(self, gene_list):
         self._MAX_TAX_DIST = -1
         self.ref_packages = {key: ts_ref_pkg.ReferencePackage() for key in gene_list}
-        self.fn = {key: [] for key in gene_list}
+        self.fn = {key: set() for key in gene_list}
         self.fp = {key: set() for key in gene_list}
         self.tp = {key: [] for key in gene_list}  # This will be a list of QuerySequence instances
         self.unlabelled_tp_query_names = {}  # These are to be added to the table at distance seven
@@ -72,7 +73,7 @@ class ConfusionTest:
         elif tool == "diamond":
             self.data_dir = output_dir + os.sep + "DIAMOND_output" + os.sep
         else:
-            logging.error("Unrecognized tool: " + tool + "\n")
+            LOGGER.error("Unrecognized tool: " + tool + "\n")
             sys.exit(1)
         return
 
@@ -139,7 +140,7 @@ class ConfusionTest:
             warn_str = ""
             for taxid in sorted(unknowns, key=int):
                 warn_str += "\t{} query sequences with unknown taxid '{}'\n".format(unknowns[taxid], taxid)
-            logging.warning("Lineage information unavailable for taxonomy IDs:\n" + warn_str)
+            LOGGER.warning("Lineage information unavailable for taxonomy IDs:\n" + warn_str)
         return
 
     def map_true_lineages(self) -> None:
@@ -154,7 +155,7 @@ class ConfusionTest:
         lineage_list = []
         info_string = "RefPkg\tName\tTaxDist\tClassified\tTrueLineage\tAssignedLineage\tOptimalLineage\n"
         for marker in self.tp:
-            for tp_inst in self.tp[marker]:  # type: QuerySequence
+            for tp_inst in self.tp[marker]:  # type: training_utils.QuerySequence
                 lineage_list.append(tp_inst.true_lineage)
                 info_string += "\t".join([marker, tp_inst.place_name, str(tp_inst.tax_dist), "True",
                                           tp_inst.true_lineage, tp_inst.assigned_lineage,
@@ -163,7 +164,7 @@ class ConfusionTest:
                 for pquery in self.unlabelled_tp_query_names[marker]:
                     info_string += "\t".join([marker, pquery.seq_name, "NA", "True", "NA", "NA", "NA"]) + "\n"
         for marker in self.fn:
-            for tp_inst in self.fn[marker]:  # type: QuerySequence
+            for tp_inst in self.fn[marker]:  # type: training_utils.QuerySequence
                 # TODO: Support headers from databases other than EggNOG
                 tax_id = tp_inst.ncbi_tax
                 try:
@@ -175,7 +176,7 @@ class ConfusionTest:
 
         with open(classification_file, 'w') as info_handler:
             info_handler.write(info_string)
-        logging.debug("Taxonomic lineage distribution of " + str(len(lineage_list)) + " 'true' reference sequences.\n")
+        LOGGER.debug("Taxonomic lineage distribution of " + str(len(lineage_list)) + " 'true' reference sequences.\n")
         input_taxa_dist = summarize_taxonomy(taxa_list=lineage_list,
                                              rank=rank,
                                              rank_depth_map=self.rank_depth_map)
@@ -183,7 +184,7 @@ class ConfusionTest:
         summary_string = ""
         for summary_lineage in sorted(input_taxa_dist, key=lambda x: input_taxa_dist[x]):
             summary_string += "\t'" + summary_lineage + "'\t" + str(input_taxa_dist[summary_lineage]) + "\n"
-        logging.debug(summary_string)
+        LOGGER.debug(summary_string)
         return
 
     def bin_true_positives_by_taxdist(self):
@@ -201,20 +202,21 @@ class ConfusionTest:
             if not refpkg.taxa_trie.rooted:
                 refpkg.taxa_trie.root_domains(root=refpkg.taxa_trie.find_root_taxon())
             self.dist_wise_tp[marker] = dict()
-            for tp_inst in self.tp[marker]:  # type: QuerySequence
+            for tp_inst in self.tp[marker]:  # type: training_utils.QuerySequence
                 # Find the optimal taxonomic assignment
                 tp_inst.true_lineage = refpkg.taxa_trie.clean_lineage_string(tp_inst.true_lineage)
                 optimal_taxon = ts_lca.optimal_taxonomic_assignment(refpkg.taxa_trie.trie, tp_inst.true_lineage)
                 if not optimal_taxon:
-                    logging.debug("Optimal taxonomic assignment '{}' for {}"
-                                  " not found in reference hierarchy.\n".format(tp_inst.true_lineage, tp_inst.place_name))
+                    LOGGER.debug("Optimal taxonomic assignment '{}' for {}"
+                                 " not found in reference hierarchy.\n".format(tp_inst.true_lineage,
+                                                                               tp_inst.place_name))
                     continue
                 tp_inst.optimal_lineage = optimal_taxon
                 tp_inst.tax_dist, status = ts_lca.compute_taxonomic_distance(tp_inst.assigned_lineage, optimal_taxon)
                 if status > 0:
-                    logging.debug("Lineages didn't converge between:\n"
-                                  "'{}' and '{}' (taxid: {})\n".format(tp_inst.assigned_lineage,
-                                                                       optimal_taxon, tp_inst.ncbi_tax))
+                    LOGGER.debug("Lineages didn't converge between:\n"
+                                 "'{}' and '{}' (taxid: {})\n".format(tp_inst.assigned_lineage,
+                                                                      optimal_taxon, tp_inst.ncbi_tax))
                 try:
                     self.dist_wise_tp[marker][tp_inst.tax_dist].append(tp_inst.place_name)
                 except KeyError:
@@ -223,13 +225,13 @@ class ConfusionTest:
 
     def check_dist(self):
         if self._MAX_TAX_DIST < 0:
-            logging.error("ConfusionTest's _MAX_TAX_DIST has yet to be set.\n")
+            LOGGER.error("ConfusionTest's _MAX_TAX_DIST has yet to be set.\n")
             sys.exit(5)
         return
 
     def check_refpkg_name(self, refpkg_name):
         if refpkg_name not in self.ref_packages:
-            logging.error(refpkg_name + " is not found in the names of markers to be tested.\n")
+            LOGGER.error(refpkg_name + " is not found in the names of markers to be tested.\n")
             sys.exit(9)
         return
 
@@ -315,7 +317,8 @@ class ConfusionTest:
 
     def bin_headers(self, assignments: dict, annot_map: dict) -> None:
         self.gather_unlabelled_queries(assignments, set(annot_map.keys()))
-        binned_tp, binned_fp, binned_fn = bin_headers(assignments, annot_map, self.entrez_query_dict, self.ref_packages)
+        binned_tp, binned_fp, binned_fn = training_utils.bin_headers(assignments, annot_map,
+                                                                     self.entrez_query_dict, self.ref_packages)
 
         self.tp.update(binned_tp)
         self.fp.update(binned_fp)
@@ -332,7 +335,7 @@ class ConfusionTest:
 
         for marker in self.fp:
             validated_fp = set()
-            for qseq in self.fp[marker]:  # type: QuerySequence
+            for qseq in self.fp[marker]:  # type: training_utils.QuerySequence
                 seq_name = qseq.place_name
                 if seq_name[0] == '>':
                     seq_name = seq_name[1:]
@@ -363,7 +366,7 @@ class ConfusionTest:
                         if homolgous_marker != marker:
                             homologous_tp_names.update(set(qs.place_name for qs in self.tp[homolgous_marker]))
 
-                for qseq in self.fn[marker]:  # type: QuerySequence
+                for qseq in self.fn[marker]:  # type: training_utils.QuerySequence
                     if qseq.place_name not in homologous_tp_names:
                         fn_names.add(qseq.place_name)
                 # Remove all sequences from this marker's false negatives that are found in a homologous TP set
@@ -395,7 +398,7 @@ class ConfusionTest:
             try:
                 internal_nodes_dict[refpkg.prefix] = refpkg.get_internal_node_leaf_map()
             except IndexError:
-                logging.error("Unable to read tree for reference package %s from '%s'.\n" % (name, refpkg.tree))
+                LOGGER.error("Unable to read tree for reference package %s from '%s'.\n" % (name, refpkg.tree))
                 sys.exit(3)
             hmm_lengths[refpkg.prefix] = refpkg.profile_length
         #
@@ -434,7 +437,7 @@ class ConfusionTest:
             if len(self.fn[marker]) == 0:
                 continue
             summary_string += "Number of false negatives for '" + marker + "':\n"
-            for qseq in self.fn[marker]:  # type: QuerySequence
+            for qseq in self.fn[marker]:  # type: training_utils.QuerySequence
                 try:
                     lineage_list.append(qseq.true_lineage)
                 except KeyError:
@@ -491,14 +494,14 @@ class ConfusionTest:
             mcc = calculate_matthews_correlation_coefficient(num_tp, num_fp, num_fn, num_tn)
             mcc_string += "\t".join([str(x) for x in [d, mcc, num_tp, num_tn, num_fp, num_fn]]) + "\n"
             d += 1
-        logging.info(mcc_string)
+        LOGGER.info(mcc_string)
         with open(mcc_table_file, 'w') as mcc_handler:
             mcc_handler.write(mcc_string)
         return
 
 
 def map_lineages(qseq_collection: set, tax_lineage_map: dict) -> None:
-    for qseq in qseq_collection:  # type: QuerySequence
+    for qseq in qseq_collection:  # type: training_utils.QuerySequence
         try:
             qseq.true_lineage = tax_lineage_map[qseq.ncbi_tax]
         except KeyError:
@@ -520,8 +523,8 @@ def summarize_taxonomy(taxa_list, rank, rank_depth_map=None):
     try:
         depth = rank_depth_map[rank] + 1
     except KeyError:
-        logging.error("Rank '{}' not present in rank-depth map:\n"
-                      "{}\n".format(rank, rank_depth_map))
+        LOGGER.error("Rank '{}' not present in rank-depth map:\n"
+                     "{}\n".format(rank, rank_depth_map))
         sys.exit(3)
 
     taxa_census = dict()
@@ -541,7 +544,7 @@ def summarize_taxonomy(taxa_list, rank, rank_depth_map=None):
         except KeyError:
             taxa_census[summary_lineage] = 1
         acc += 1
-    logging.debug(str(empty) + " empty lineages encountered.\n")
+    LOGGER.debug(str(empty) + " empty lineages encountered.\n")
 
     return taxa_census
 
@@ -580,6 +583,8 @@ def get_arguments(sys_args):
     parser.optopt.add_argument("--tool", default="treesapp", required=False,
                                choices=["treesapp", "graftm", "diamond"],
                                help="Classify using one of the tools: treesapp [DEFAULT], graftm, or diamond.")
+    parser.optopt.add_argument("--gpkg_dir", default=None, required=False, dest="gpkg_path",
+                               help="Path to a directory containing GraftM reference packages")
 
     ts.add_argument("--svm", default=False, required=False, action="store_true",
                     help="Uses the support vector machine (SVM) classification filter. "
@@ -591,7 +596,7 @@ def get_arguments(sys_args):
         args.output += os.sep
 
     if len(args.input) > 1:
-        logging.error("Unable to handle more than one fastx_input file.\n")
+        LOGGER.error("Unable to handle more than one fastx_input file.\n")
         sys.exit(7)
     args.input = args.input.pop(0)
 
@@ -599,7 +604,7 @@ def get_arguments(sys_args):
 
 
 def validate_command(args, sys_args):
-    logging.debug("Command used:\n" + ' '.join(sys_args) + "\n")
+    LOGGER.debug("Command used:\n" + ' '.join(sys_args) + "\n")
 
     if not os.path.isdir(args.output):
         os.mkdir(args.output)
@@ -609,20 +614,25 @@ def validate_command(args, sys_args):
         args.refpkg_dir = args.treesapp + "data" + os.sep
 
     if sys.version_info < (2, 9):
-        logging.error("Python version '" + str(sys.version_info) + "' not supported.\n")
+        LOGGER.error("Python version '" + str(sys.version_info) + "' not supported.\n")
         sys.exit(3)
 
-    if args.refpkg_dir and args.refpkg_dir[-1] != os.sep:
+    if not args.refpkg_dir:
+        args.refpkg_dir = args.treesapp + "data"
+    if args.refpkg_dir[-1] != os.sep:
         args.refpkg_dir += os.sep
     if args.tool in ["diamond", "graftm"]:
-        if not args.refpkg_dir:
-            logging.error(args.tool + " specified but a GraftM reference package directory was not provided.\n")
+        if not args.gpkg_path:
+            LOGGER.error(args.tool + " specified but a GraftM reference package directory was not provided.\n")
             sys.exit(17)
-        elif not os.path.isdir(args.refpkg_dir):
-            logging.error(args.refpkg_dir + " GraftM reference package directory does not exist!\n")
+        if not os.path.isdir(args.gpkg_path):
+            LOGGER.error(args.gpkg_path + " GraftM reference package directory does not exist!\n")
             sys.exit(17)
-        elif len(glob(args.refpkg_dir + "*gpkg")) == 0:
-            logging.error("No GraftM reference packages found in " + args.refpkg_dir + ".\n")
+
+        if args.gpkg_path[-1] != os.sep:
+            args.gpkg_path += os.sep
+        if len(glob(args.gpkg_path + "*gpkg")) == 0:
+            LOGGER.error("No GraftM reference packages found in " + args.gpkg_path + ".\n")
             sys.exit(17)
 
     if args.targets:
@@ -639,7 +649,7 @@ def calculate_matthews_correlation_coefficient(tp: int, fp: int, fn: int, tn: in
     if numerator == 0 or denominator == 0:
         return 0.0
     else:
-        return round(numerator/sqrt(denominator), 3)
+        return round(numerator / sqrt(denominator), 3)
 
 
 def check_previous_output(output_dir: str, files: list, overwrite=False) -> None:
@@ -647,7 +657,7 @@ def check_previous_output(output_dir: str, files: list, overwrite=False) -> None
         os.mkdir(output_dir)
     elif overwrite:
         if os.path.isdir(output_dir):
-            logging.warning("Overwriting directory '{}' in 5 seconds. Press Ctrl-C to cancel.\n".format(output_dir))
+            LOGGER.warning("Overwriting directory '{}' in 5 seconds. Press Ctrl-C to cancel.\n".format(output_dir))
             sleep(5)
             shutil.rmtree(output_dir)
         os.mkdir(output_dir)
@@ -677,11 +687,11 @@ def filter_redundant_og(query_og_map: dict) -> set:
             redundants.add(query_name)
 
     if redundants:
-        logging.warning("{}/{} query sequences were annotated as multiple reference packages and have been removed.\n"
-                        "".format(len(redundants), len(query_og_map)))
+        LOGGER.warning("{}/{} query sequences were annotated as multiple reference packages and have been removed.\n"
+                       "".format(len(redundants), len(query_og_map)))
         for og in sorted(ortho_filters):
-            logging.info("{}% of annotated sequences were filtered for '{}'.\n"
-                         "".format(round((ortho_filters[og]*100)/ortho_counts[og], 1), og))
+            LOGGER.info("{}% of annotated sequences were filtered for '{}'.\n"
+                        "".format(round((ortho_filters[og] * 100) / ortho_counts[og], 1), og))
 
     for query_name in redundants:
         query_og_map.pop(query_name)
@@ -747,10 +757,10 @@ def match_queries_to_refpkgs(query_name_map: dict, refpkg_dict: dict) -> (dict,
         query_to_code.pop(qname)
 
     if missing_matches:
-        logging.info("{} unique reference packages ({} total) referred to in the annotation file were not found"
-                     " in the reference package list.\n".format(len(set(missing_matches)), len(missing_matches)))
-        logging.debug("Unique annotation names that couldn't be mapped to reference packages:\n{}\n"
-                      "".format(set(missing_matches)))
+        LOGGER.info("{} unique reference packages ({} total) referred to in the annotation file were not found"
+                    " in the reference package list.\n".format(len(set(missing_matches)), len(missing_matches)))
+        LOGGER.debug("Unique annotation names that couldn't be mapped to reference packages:\n{}\n"
+                     "".format(set(missing_matches)))
 
     return query_to_prefix, query_to_code
 
@@ -766,10 +776,10 @@ def mcc_calculator(sys_args):
     classification_info_output = output_prefix + "_classifications.tsv"
     graftm_exe = ""
 
-    # Instantiate the logging instance and write the log
-    prep_logging(log_name, args.verbose)
+    # Instantiate the LOGGER instance and write the log
+    logger.prep_logging(log_name, args.verbose)
 
-    logging.info("\n##\t\t\tBeginning Matthews Correlation Coefficient analysis\t\t\t##\n")
+    LOGGER.info("\n##\t\t\tBeginning Matthews Correlation Coefficient analysis\t\t\t##\n")
     validate_command(args, sys.argv)
 
     ##
@@ -788,7 +798,7 @@ def mcc_calculator(sys_args):
     test_obj.redundant_queries = filter_redundant_og(query_name_dict)
     query_to_prefix, query_to_code = match_queries_to_refpkgs(query_name_dict, refpkg_dict)
     if len(query_to_code) == 0 or len(query_to_prefix) == 0:
-        logging.error("Matching reference package annotations to queries failed.\n")
+        LOGGER.error("Matching reference package annotations to queries failed.\n")
         sys.exit(13)
 
     refpkg_name_query_map = create_refpkg_query_map(query_to_code, refpkg_dict)
@@ -802,22 +812,18 @@ def mcc_calculator(sys_args):
     ##
     # Load the taxonomic trie for each reference package
     ##
-    if args.tool == "treesapp":
-        test_obj.ref_packages = refpkg_dict
-    elif args.tool in ["graftm", "diamond"]:
+    test_obj.ref_packages = refpkg_dict
+    if args.tool in ["graftm", "diamond"]:
         graftm_exe = utilities.fetch_executable_path("graftM", test_obj.treesapp_dir)
-        for gpkg in glob(args.refpkg_dir + "*gpkg"):
-            gpkg_name = str(os.path.basename(gpkg).split('.')[0])
+        for gpkg in glob(args.gpkg_path + "*gpkg"):
+            gpkg_name = graftm_utils.get_graftm_pkg_name(gpkg)
             if gpkg_name in test_obj.ref_packages:
                 try:
                     tax_ids_file = glob(os.path.join(gpkg, gpkg_name + ".gpkg.refpkg",
                                                      gpkg_name + "*taxonomy.csv")).pop()
-                    test_obj.ref_packages[gpkg_name].taxa_trie = file_parsers.grab_graftm_taxa(tax_ids_file)
+                    test_obj.ref_packages[gpkg_name].taxa_trie = graftm_utils.grab_graftm_taxa(tax_ids_file)
                 except IndexError:
-                    logging.warning("No GraftM taxonomy file found for {}. Is this gpkg complete?\n".format(gpkg_name))
-    else:
-        logging.error("Unrecognized tool name '{}'\n".format(args.tool))
-        sys.exit(3)
+                    LOGGER.warning("No GraftM taxonomy file found for {}. Is this gpkg complete?\n".format(gpkg_name))
 
     ##
     # Run the specified taxonomic analysis tool and collect the classifications
@@ -826,54 +832,55 @@ def mcc_calculator(sys_args):
     test_fa_prefix = '.'.join(os.path.basename(args.input).split('.')[:-1])
     classification_lines = []
     if args.tool == "treesapp":
-        ref_pkgs = ','.join(test_obj.ref_packages.keys())
         classification_table = os.path.join(args.output, "TreeSAPP_output", "final_outputs", "classifications.tsv")
         if not os.path.isfile(classification_table):
             classify_args = ["-i", args.input,
-                             "-t", ref_pkgs,
+                             "-t", ','.join(test_obj.ref_packages.keys()),
                              "-n", str(args.num_threads),
                              "-m", "prot",
                              "--output", test_obj.data_dir,
                              "--stringency", args.stringency,
                              "--hmm_coverage", str(args.hmm_coverage),
+                             "--query_coverage", str(args.query_coverage),
                              "--placement_summary", args.p_sum,
                              "--min_like_weight_ratio", str(args.min_lwr),
-                             "--max_pendant_length", str(args.max_pd),
                              "--overwrite", "--delete"]
             if args.trim_align:
                 classify_args.append("--trim_align")
             if args.svm:
                 classify_args.append("--svm")
+            if args.max_pd:
+                classify_args += ["--max_pendant_length", str(args.max_pd)]
+            if args.max_evo:
+                classify_args += ["--max_evol_distance", str(args.max_evo)]
             assign.assign(classify_args)
         classification_lines = file_parsers.read_classification_table(classification_table)
         assignments = assignments_to_pqueries(classification_lines)
     else:
         # Since you are only able to analyze a single reference package at a time with GraftM, this is ran iteratively
-        for gpkg in glob(args.refpkg_dir + "*gpkg"):
-            pkg_name = str(os.path.basename(gpkg).split('.')[0])
-            if not pkg_name:
-                logging.error("Unable to parse marker name from gpkg '{}'\n".format(gpkg))
-                sys.exit(5)
+        for gpkg in glob(args.gpkg_path + "*gpkg"):
+            pkg_name = graftm_utils.get_graftm_pkg_name(gpkg)
             if pkg_name not in test_obj.ref_packages:
-                logging.warning("'{}' not in {} and will be skipped...\n".format(pkg_name, args.annot_map))
+                LOGGER.warning("'{}' not in {} and will be skipped...\n".format(pkg_name, args.annot_map))
                 continue
             output_dir = test_obj.data_dir + pkg_name + os.sep
             classification_table = output_dir + test_fa_prefix + os.sep + test_fa_prefix + "_read_tax.tsv"
             if not os.path.isfile(classification_table):
-                wrapper.run_graftm_graft(graftm_exe, args.input, output_dir,
-                                         gpkg_path=gpkg, classifier=args.tool, num_threads=args.num_threads)
+                graftm_utils.run_graftm_graft(args.input, output_dir,
+                                              graftm_exe=graftm_exe, gpkg_path=gpkg,
+                                              classifier=args.tool, num_threads=args.num_threads)
 
             # TODO: Figure out how to convert GraftM classifications into JPlace objects
-            assignments[pkg_name] = file_parsers.read_graftm_classifications(classification_table)
+            assignments[pkg_name] = graftm_utils.read_graftm_classifications(classification_table)
 
     if len(assignments) == 0:
-        logging.error("No sequences were classified by " + args.tool + ".\n")
+        LOGGER.error("No sequences were classified by " + args.tool + ".\n")
         sys.exit(3)
 
-    logging.info("Reading headers in " + args.input + "... ")
+    LOGGER.info("Reading headers in " + args.input + "... ")
     test_obj.all_queries = set([seq_name[1:] if seq_name[0] == '>' else seq_name for
                                 seq_name in get_headers(args.input)])
-    logging.info("done.\n")
+    LOGGER.info("done.\n")
 
     ##
     # Bin the test sequence names into their respective confusion categories (TP, TN, FP, FN)
@@ -887,15 +894,15 @@ def mcc_calculator(sys_args):
 
     test_obj.summarise_reference_taxa(taxa_dist_output, classification_info_output, summary_rank)
     if args.tool == "treesapp" and classification_lines:
-        logging.debug(test_obj.summarise_type_one_placements(classification_lines))
-    logging.debug(test_obj.summarize_type_two_taxa(summary_rank))
-    logging.debug(test_obj.true_positive_taxonomic_summary(summary_rank, True))
+        LOGGER.debug(test_obj.summarise_type_one_placements(classification_lines))
+    LOGGER.debug(test_obj.summarize_type_two_taxa(summary_rank))
+    LOGGER.debug(test_obj.true_positive_taxonomic_summary(summary_rank, True))
 
     ##
     # Report the MCC score across different taxonomic distances - should increase with greater allowed distance
     ##
     test_obj._MAX_TAX_DIST = 6
-    logging.debug(test_obj.get_info(True))
+    LOGGER.debug(test_obj.get_info(True))
     test_obj.tabularise_mcc_stats(mcc_file)
     return
 
