@@ -1,28 +1,19 @@
-#!/usr/bin/env python3
-
-__author__ = 'Connor Morgan-Lang'
-
 import sys
 import re
 import os
 import logging
 
 from collections import namedtuple
+from matplotlib import colors as mpl_colours
 import seaborn
 
 from treesapp.phylo_seq import TreeLeafReference
 from treesapp.classy import TreeSAPP
 from treesapp.refpkg import ReferencePackage
 from treesapp.taxonomic_hierarchy import Taxon
+from treesapp import logger
 
-
-class Clade:
-    def __init__(self):
-        self.taxon = ""
-        self.colour = ""
-        self.leaves = []  # Leaf names (not internal nodes)
-        self.i_nodes = []  # List of all internal nodes that
-        return
+LOGGER = logging.getLogger(logger.logger_name())
 
 
 class PhyPainter(TreeSAPP):
@@ -33,11 +24,14 @@ class PhyPainter(TreeSAPP):
         self.taxa_to_colour = set()
         self.rank = ""
         self.palette = ""
+        self.unknown_label = None
+        self.unknown_col = ""
         self.set_op = ""
         self.feature_name = ""
         self.num_taxa = 0
         self.num_seqs = 0
         self.rank_depth = -1
+        self.order_method = "phylo"
 
     def primer(self, args) -> None:
         rank_map = {}
@@ -51,7 +45,7 @@ class PhyPainter(TreeSAPP):
             if not rank_map:
                 rank_map = ref_pkg.taxa_trie.accepted_ranks_depths
 
-        logging.debug("\tGenerating colour palette for reference package(s): {}.\n"
+        self.ts_logger.debug("\tGenerating colour palette for reference package(s): {}.\n"
                       "".format(', '.join(self.refpkg_dict.keys())))
 
         self.set_op = args.set_op
@@ -61,15 +55,10 @@ class PhyPainter(TreeSAPP):
             try:
                 os.makedirs(self.output_dir)
             except IOError:
-                logging.error("Unable to create output directory '{}'.\n".format(self.output_dir))
+                self.ts_logger.error("Unable to create output directory '{}'.\n".format(self.output_dir))
                 sys.exit(1)
 
-        if args.rank not in rank_map:
-            logging.error("Rank '{}' not accepted! Please choose one of the following:\n"
-                          "{}\n".format(args.rank, ", ".join(rank_map.keys())))
-            sys.exit(1)
-        else:
-            self.rank = args.rank
+        self.rank = args.rank
         self.palette = args.palette
 
         # Determine the prefix for the colour files
@@ -78,7 +67,21 @@ class PhyPainter(TreeSAPP):
         else:
             self.feature_name = args.attribute
             self.rank = None
+            self.order_method = "alpha"
 
+        if args.un_col:
+            self.unknown_label = "Unknown"
+            self.set_unknown_colour_from_mpl(args.un_col)
+
+        return
+
+    def set_unknown_colour_from_mpl(self, requested_colour: str):
+        try:
+            self.unknown_col = mpl_colours.get_named_colors_mapping()[requested_colour]
+        except KeyError:
+            self.ts_logger.error("Colour '{}' is not available in matplotlib.colors.get_named_colors_mapping().\n"
+                          "Unable to find hexcode for requested unknown colour.\n")
+            sys.exit(11)
         return
 
     def find_rank_depth(self, ref_pkg: ReferencePackage, depth: int) -> None:
@@ -109,11 +112,11 @@ class PhyPainter(TreeSAPP):
             elif lineage_path[0] in luca_reps:
                 offsets.add(depth)
             else:
-                logging.debug("Unable to find root or domain name for '%s'. Skipping.\n" % leaf.lineage)
+                self.ts_logger.debug("Unable to find root or domain name for '%s'. Skipping.\n" % leaf.lineage)
                 continue
 
         if len(offsets) > 1:
-            logging.error("Inconsistent rank positions encountered across lineages.\n")
+            self.ts_logger.error("Inconsistent rank positions encountered across lineages.\n")
             sys.exit(13)
         self.rank_depth = offsets.pop()
 
@@ -142,7 +145,7 @@ class PhyPainter(TreeSAPP):
                     break
             taxon_leaf_map.pop(taxon)
 
-        logging.info("Remaining sequences for colouring\t{}\n"
+        self.ts_logger.info("Remaining sequences for colouring\t{}\n"
                      "Remaining taxa for colouring\t{}\n"
                      .format(self.num_seqs, self.num_taxa))
         return
@@ -176,7 +179,7 @@ class PhyPainter(TreeSAPP):
         filter_str += ", ".join(filtered_taxa) + "\n"
         filter_str += "Sequences from unwanted taxa removed\t" + str(filtered_seqs) + "\n"
         filter_str += "Unwanted taxa removed\t" + str(len(filtered_taxa)) + "\n"
-        logging.info(filter_str)
+        LOGGER.info(filter_str)
 
         return filtered_taxa
 
@@ -192,7 +195,7 @@ class PhyPainter(TreeSAPP):
         if not filtered_taxa_strings:
             filtered_taxa_strings.append("\tNone")
 
-        logging.info("Taxa filtered due to proportion < {}:\n"
+        LOGGER.info("Taxa filtered due to proportion < {}:\n"
                      "\t{}\nLow-abundance taxa removed\t{}\n"
                      "".format(min_p, "\n\t".join(filtered_taxa_strings), len(filtered_taxa)))
 
@@ -215,15 +218,23 @@ class PhyPainter(TreeSAPP):
                 else:
                     pass
             if not ancestral_node:
-                logging.warning("Unable to find internal node ancestral to all children of " + taxon + ".\n")
+                LOGGER.warning("Unable to find internal node ancestral to all children of " + taxon + ".\n")
                 bad_taxa.add(taxon)
                 continue
             if len(internal_node_map[ancestral_node]) > len(leaf_nodes):
-                logging.warning(taxon + " clade was found to be polyphyletic.\n")
+                LOGGER.warning(taxon + " clade was found to be polyphyletic.\n")
                 bad_taxa.add(taxon)
 
         return bad_taxa
 
+    def add_unknowns_to_feature_leaf_map(self, feature_leaf_map: dict, ref_pkg: ReferencePackage) -> None:
+        # Assign 'Unknown' feature to all leaves missing from the taxa_leaf_nodes_map if set
+        if self.unknown_col:
+            present = set(sum(feature_leaf_map.values(), []))
+            absent = set(ref_pkg.ref_names()).difference(present)
+            feature_leaf_map[self.unknown_label] = list(absent)
+        return
+
     @staticmethod
     def find_mono_clades(taxon_leaf_map: dict, ref_pkg: ReferencePackage) -> dict:
         """
@@ -234,12 +245,19 @@ class PhyPainter(TreeSAPP):
         :return: A dictionary mapping taxon names in taxon_leaf_map to the minimal set of internal nodes in the
          ReferencePackage's tree that cover all leaf nodes in the taxon_leaf_map[taxon].
         """
-        taxa_inode_map = {}
+        taxa_leaf_nodes_map = {}
+
         for taxon, taxon_leaves in taxon_leaf_map.items():
-            taxa_inode_map[taxon] = ref_pkg.get_monophyletic_clades(taxon_name=taxon, leaf_nodes=set(taxon_leaves))
-        return taxa_inode_map
+            taxa_leaf_nodes_map[taxon] = ref_pkg.get_monophyletic_clades(taxon_name=taxon, leaf_nodes=set(taxon_leaves))
+        return taxa_leaf_nodes_map
 
     def harmonize_taxa_colours(self, taxon_leaf_map: dict, set_operation: str) -> None:
+        """
+
+        :param taxon_leaf_map:
+        :param set_operation:
+        :return:
+        """
         if not self.taxa_to_colour:
             self.taxa_to_colour.update(set(taxon_leaf_map.keys()))
             return
@@ -249,7 +267,7 @@ class PhyPainter(TreeSAPP):
         elif set_operation == 'i':
             self.taxa_to_colour.intersection(set(taxon_leaf_map.keys()))
         else:
-            logging.error("Unsupported set operation specified: '{}'\n".format(set_operation))
+            self.ts_logger.error("Unsupported set operation specified: '{}'\n".format(set_operation))
             sys.exit(17)
 
         return
@@ -257,24 +275,50 @@ class PhyPainter(TreeSAPP):
     def get_colours(self) -> list:
         n_clade = len(self.taxa_to_colour)
         if n_clade <= 1 and self.rank:
-            logging.warning("{} unique clade(s) identified at rank '{}'."
+            self.ts_logger.warning("{} unique clade(s) identified at rank '{}'."
                             " Consider using a more resolved rank or adjusting other filtering parameters.\n"
                             "".format(n_clade, self.feature_name))
         else:
-            logging.debug("Identified {} unique clades for '{}'.\n".format(n_clade, self.feature_name))
+            self.ts_logger.debug("Identified {} unique clades for '{}'.\n".format(n_clade, self.feature_name))
 
         try:
             colours = seaborn.color_palette(self.palette, n_clade).as_hex()
-            logging.info("Using palette '{}' for styling.\n".format(self.palette))
+            self.ts_logger.info("Using palette '{}' for styling.\n".format(self.palette))
         except ValueError as e:
-            logging.error(str(e) + "\n")
+            self.ts_logger.error(str(e) + "\n")
             sys.exit(4)
 
         if len(colours) != n_clade:
-            logging.error("Bad colour parsing! len(colours) != number of clades.\n")
+            self.ts_logger.error("Bad colour parsing! len(colours) != number of clades.\n")
             raise ValueError
         return colours
 
+    def map_colours_to_taxa(self, taxa_order: dict):
+        """
+        Function for mapping
+
+        :param taxa_order: Dictionary indexed by numerical order of the taxa
+        :return:
+        """
+
+        colours = self.get_colours()
+        palette_taxa_map = dict()
+        alpha_sort_i = 0
+
+        if len(taxa_order) != len(colours):
+            self.ts_logger.error("Number of taxa (%d) and colours (%d) are not equal!\n" % (len(taxa_order), len(colours)))
+            sys.exit(7)
+
+        for taxon_num in sorted(taxa_order, key=int):
+            taxon = taxa_order[taxon_num]
+            palette_taxa_map[taxon] = colours[alpha_sort_i]
+            alpha_sort_i += 1
+
+        if self.unknown_label and self.unknown_col:
+            palette_taxa_map[self.unknown_label] = self.unknown_col
+
+        return palette_taxa_map
+
 
 def convert_clades_to_ranges(taxa_clades: dict, leaf_order: list) -> dict:
     taxa_ranges = dict()
@@ -294,7 +338,7 @@ def create_write_file(file_name: str, text: str) -> None:
     try:
         file_handler = open(file_name, 'w')
     except IOError:
-        logging.error("Unable to open file '{}' for writing!\n".format(file_name))
+        LOGGER.error("Unable to open file '{}' for writing!\n".format(file_name))
         raise IOError
 
     # Write the iTOL-formatted header and node, colour and taxon fields
@@ -303,33 +347,12 @@ def create_write_file(file_name: str, text: str) -> None:
     return
 
 
-def map_colours_to_taxa(taxa_order, colours):
-    """
-    Function for mapping
-
-    :param taxa_order: Dictionary indexed by numerical order of the taxa
-    :param colours:
-    :return:
-    """
-    palette_taxa_map = dict()
-    alpha_sort_i = 0
-
-    if len(taxa_order) != len(colours):
-        logging.error("Number of taxa (%d) and colours (%d) are not equal!\n" % (len(taxa_order), len(colours)))
-        sys.exit(7)
-
-    for taxon_num in sorted(taxa_order, key=int):
-        taxon = taxa_order[taxon_num]
-        palette_taxa_map[taxon] = colours[alpha_sort_i]
-        alpha_sort_i += 1
-
-    return palette_taxa_map
-
-
 def write_colours_styles(taxon_leaf_map: dict, palette_taxa_map: dict, style_output: str) -> None:
     colourless = set()
 
-    colours_style_string = "TREE_COLORS\nSEPARATOR TAB\nDATA\n"
+    colours_style_string = "TREE_COLORS\n" \
+                           "SEPARATOR TAB\n" \
+                           "DATA\n"
     for taxon, col in palette_taxa_map.items():
         try:
             leaves = taxon_leaf_map[taxon]
@@ -339,19 +362,24 @@ def write_colours_styles(taxon_leaf_map: dict, palette_taxa_map: dict, style_out
             colours_style_line = [str(leaf_num), "range", col, taxon]
             if len(colours_style_line) > 0:
                 colours_style_string += "\t".join(colours_style_line) + "\n"
-    # TODO: Find new way to track the uncoloured leaves
 
-    logging.info("Output is available in " + style_output + ".\n")
-    logging.debug("{} lineages were not assigned to a colour:\n\t{}\n".format(len(colourless), "\n\t".join(colourless)))
+    LOGGER.info("iTOL colours style input written to " + style_output + ".\n")
+    LOGGER.debug("{} lineages were not assigned to a colour:\n\t{}\n".format(len(colourless), "\n\t".join(colourless)))
 
     create_write_file(style_output, colours_style_string)
 
     return
 
 
-def write_colour_strip(taxa_nodes: dict, palette_taxa_map: dict, colour_strip_file: str) -> None:
-    colour_strip_text = "DATASET_COLORSTRIP\nSEPARATOR SPACE\nDATASET_LABEL clade_colours\n" +\
-                        "STRIP_WIDTH 75\nMARGIN 0\nSHOW_INTERNAL 1\nDATA\n"
+def write_colour_strip(taxa_nodes: dict, palette_taxa_map: dict, colour_strip_file: str, data_label: str) -> None:
+    colour_strip_text = "DATASET_COLORSTRIP\n" \
+                        "SEPARATOR SPACE\n" \
+                        "DATASET_LABEL {}\n".format(data_label) +\
+                        "STRIP_WIDTH 75\n" \
+                        "BORDER_WIDTH 1" \
+                        "MARGIN 10\n" \
+                        "SHOW_INTERNAL 1\n" \
+                        "DATA\n"
     for taxon, col in palette_taxa_map.items():
         try:
             leaf_ranges = taxa_nodes[taxon]
@@ -362,7 +390,7 @@ def write_colour_strip(taxa_nodes: dict, palette_taxa_map: dict, colour_strip_fi
             if len(colours_style_line) > 0:
                 colour_strip_text += " ".join(colours_style_line) + "\n"
 
-    logging.info("Output is available in " + colour_strip_file + ".\n")
+    LOGGER.info("iTOL colour strip input written to " + colour_strip_file + ".\n")
 
     create_write_file(colour_strip_file, colour_strip_text)
     return
@@ -380,28 +408,34 @@ def clade_leaf_sides(clade_leaves, leaf_order: list) -> namedtuple:
     return p
 
 
-def order_taxa(taxa_to_colour: set, taxon_leaf_map: dict, leaf_order: list):
+def order_taxa(taxa_to_colour: set, taxon_leaf_map: dict, leaf_order: list, method="phylo"):
     # TODO: improve this function to look at mean/median distances from the left-most taxon
     leftmost_taxon = dict()
     taxon_order = dict()
     indices = dict()  # Could just use a list but may as well keep track of the left-most node with dict
-    for taxon, leaves in taxon_leaf_map.items():
-        if taxon not in taxa_to_colour:
-            continue
-        for leaf_name in leaves:
-            indices[int(leaf_order.index(leaf_name))] = leaf_name
-        leftmost_leaf = min(indices.keys())
-        leftmost_taxon[leftmost_leaf] = taxon
-        indices.clear()
-    # Collect the order of the taxa in the phylogeny by their left-most positions
-    i = 0
-    for index in sorted(leftmost_taxon):
-        taxon_order[i] = leftmost_taxon[index]
-        i += 1
-    # Ensure all of the taxa that need to be coloured are in taxon_order
-    for taxon in taxa_to_colour.difference(set(leftmost_taxon.values())):
-        taxon_order[i] = taxon
-        i += 1
+    if method == "phylo":
+        for taxon, leaves in taxon_leaf_map.items():
+            if taxon not in taxa_to_colour:
+                continue
+            for leaf_name in leaves:
+                indices[int(leaf_order.index(leaf_name))] = leaf_name
+            leftmost_leaf = min(indices.keys())
+            leftmost_taxon[leftmost_leaf] = taxon
+            indices.clear()
+        # Collect the order of the taxa in the phylogeny by their left-most positions
+        i = 0
+        for index in sorted(leftmost_taxon):
+            taxon_order[i] = leftmost_taxon[index]
+            i += 1
+        # Ensure all of the taxa that need to be coloured are in taxon_order
+        for taxon in taxa_to_colour.difference(set(leftmost_taxon.values())):
+            taxon_order[i] = taxon
+            i += 1
+    elif method == "alpha":
+        i = 0
+        for feature in sorted(taxa_to_colour):
+            taxon_order[i] = feature
+            i += 1
     return taxon_order
 
 
